'''
Function:
    qq音乐下载: https://y.qq.com/
Author:
    Charles
微信公众号:
    Charles的皮卡丘
'''
import base64
import random
import requests
from .base import Base
from ..utils.misc import *


'''QQ音乐下载类'''
class QQMusic(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(QQMusic, self).__init__(config, logger_handle, **kwargs)
        self.source = 'qqmusic'
        self.__initialize()
    '''歌曲搜索'''
    def search(self, keyword):
        self.logger_handle.info('正在%s中搜索 ——> %s...' % (self.source, keyword))
        cfg = self.config.copy()
        params = {
            'w': keyword,
            'format': 'json',
            'p': '1',
            'n': cfg['search_size_per_source']
        }
        response = self.session.get(self.search_url, headers=self.headers, params=params)
        all_items = response.json()['data']['song']['list']
        songinfos = []
        for item in all_items:
            params = {
                'guid': str(random.randrange(1000000000, 10000000000)),
                'loginUin': '3051522991',
                'format': 'json',
                'platform': 'yqq',
                'cid': '205361747',
                'uin': '3051522991',
                'songmid': item['songmid'],
                'needNewCode': '0'
            }
            ext = ''
            download_url = ''
            filesize = '-MB'
            for quality in [("A000", "ape", 800), ("F000", "flac", 800), ("M800", "mp3", 320), ("C400", "m4a", 128), ("M500", "mp3", 128)]:
                params['filename'] = '%s%s.%s' % (quality[0], item['songmid'], quality[1])
                response = self.session.get(self.mobile_fcg_url, headers=self.ios_headers, params=params)
                response_json = response.json()
                if response_json['code'] != 0: continue
                vkey = response_json.get('data', {}).get('items', [{}])[0].get('vkey', '')
                if vkey:
                    ext = quality[1]
                    download_url = 'http://dl.stream.qqmusic.qq.com/{}?vkey={}&guid={}&uin=3051522991&fromtag=64'.format('%s%s.%s' % (quality[0], item['songmid'], quality[1]), vkey, params['guid'])
                    if ext in ['ape', 'flac']:
                        filesize = item['size%s' % ext]
                    elif ext in ['mp3', 'm4a']:
                        filesize = item['size%s' % quality[-1]]
                    break
            if not download_url:
                params = {
                    'data': json.dumps({
                        "req": {"module": "CDN.SrfCdnDispatchServer", "method": "GetCdnDispatch", "param": {"guid": "3982823384", "calltype": 0, "userip": ""}},
                        "req_0": {"module": "vkey.GetVkeyServer", "method": "CgiGetVkey", "param": {"guid": "3982823384", "songmid": [item['songmid']], "songtype": [0], "uin": "0", "loginflag": 1, "platform": "20"}},
                        "comm": {"uin": 0, "format": "json", "ct": 24, "cv": 0}
                    })
                }
                response = self.session.get(self.fcg_url, headers=self.ios_headers, params=params)
                response_json = response.json()
                if response_json['code'] == 0 and response_json['req']['code'] == 0 and response_json['req_0']['code'] == 0:
                    ext = '.m4a'
                    download_url = str(response_json["req"]["data"]["freeflowsip"][0]) + str(response_json["req_0"]["data"]["midurlinfo"][0]["purl"])
                    filesize = item['size128']
            if (not download_url) or (filesize == '-MB') or (filesize == 0): continue
            params = {
                'songmid': str(item['songmid']),
                'g_tk': '5381',
                'loginUin': '0',
                'hostUin': '0',
                'format': 'json',
                'inCharset': 'utf8',
                'outCharset': 'utf-8',
                'platform': 'yqq'
            }
            response = self.session.get(self.lyric_url, headers={'Referer': 'https://y.qq.com/portal/player.html'}, params=params)
            lyric = base64.b64decode(response.json().get('lyric', '')).decode('utf-8')
            filesize = str(round(filesize/1024/1024, 2)) + 'MB'
            duration = int(item.get('interval', 0))
            songinfo = {
                'source': self.source,
                'songid': str(item['songmid']),
                'singers': filterBadCharacter(','.join([s.get('name', '') for s in item.get('singer', [])])),
                'album': filterBadCharacter(item.get('albumname', '-')),
                'songname': filterBadCharacter(item.get('songname', '-')),
                'savedir': cfg['savedir'],
                'savename': '_'.join([self.source, filterBadCharacter(item.get('songname', '-'))]),
                'download_url': download_url,
                'lyric': lyric,
                'filesize': filesize,
                'ext': ext,
                'duration': seconds2hms(duration)
            }
            if not songinfo['album']: songinfo['album'] = '-'
            songinfos.append(songinfo)
        return songinfos
    '''初始化'''
    def __initialize(self):
        self.ios_headers = {
            'User-Agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 9_1 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13B143 Safari/601.1',
            'Referer': 'http://y.qq.com'
        }
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36',
            'Referer': 'http://y.qq.com'
        }
        self.search_url = 'https://c.y.qq.com/soso/fcgi-bin/client_search_cp'
        self.mobile_fcg_url = 'https://c.y.qq.com/base/fcgi-bin/fcg_music_express_mobile3.fcg'
        self.fcg_url = 'https://u.y.qq.com/cgi-bin/musicu.fcg'
        self.lyric_url = 'https://c.y.qq.com/lyric/fcgi-bin/fcg_query_lyric_new.fcg'


'''
Function:
    荔枝FM下载: http://m.lizhi.fm
Author:
    Charles
微信公众号:
    Charles的皮卡丘
'''
import requests
from .base import Base
from ..utils.misc import *


'''荔枝FM下载类'''
class Lizhi(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(Lizhi, self).__init__(config, logger_handle, **kwargs)
        self.source = 'lizhi'
        self.__initialize()
    '''歌曲搜索'''
    def search(self, keyword):
        self.logger_handle.info('正在%s中搜索 ——> %s...' % (self.source, keyword))
        cfg = self.config.copy()
        response = self.session.get(self.search_url.format(keyword), headers=self.headers)
        all_items = response.json()['audio']['data']        
        songinfos = []
        for item in all_items:
            response = self.session.get(self.songinfo_url.format(item['audio']['id']), headers=self.headers)
            response_json = response.json()
            if response_json['code'] != 0: continue
            download_url = response_json['data'].get('userVoice', {}).get('voicePlayProperty', {}).get('trackUrl', '')
            if not download_url: continue
            filesize = '-'
            ext = download_url.split('.')[-1]
            duration = int(item.get('audio').get('duration', 0))
            songinfo = {
                'source': self.source,
                'songid': str(item['audio']['id']),
                'singers': filterBadCharacter(item['radio'].get('user_name', '-')),
                'album': filterBadCharacter(item['radio'].get('name', '-')),
                'songname': filterBadCharacter(item['audio'].get('name', '-')),
                'savedir': cfg['savedir'],
                'savename': '_'.join([self.source, filterBadCharacter(item['audio'].get('name', '-'))]),
                'download_url': download_url,
                'lyric': '',
                'filesize': filesize,
                'ext': ext,
                'duration': seconds2hms(duration)
            }
            if not songinfo['album']: songinfo['album'] = '-'
            songinfos.append(songinfo)
            if len(songinfos) == cfg['search_size_per_source']: break
        return songinfos
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 9_1 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13B143 Safari/601.1',
            'Referer': 'http://m.lizhi.fm'
        }
        self.search_url = 'http://m.lizhi.fm/api/search_audio/{}/1'
        self.songinfo_url = 'https://m.lizhi.fm/vodapi/voice/info/{}'


'''
Function:
    音乐下载器
Author:
    Charles
微信公众号:
    Charles的皮卡丘
'''
import os
import sys
import copy
import time
import threading
if __name__ == '__main__':
    from modules import *
    from __init__ import __version__
else:
    from .modules import *
    from .__init__ import __version__


'''basic info'''
BASICINFO = '''************************************************************
Function: 音乐下载器 V%s
Author: Charles
微信公众号: Charles的皮卡丘
操作帮助:
    输入r: 重新初始化程序(即返回主菜单)
    输入q: 退出程序
    下载多首歌曲: 选择想要下载的歌曲时,输入{1,2,5}可同时下载第1,2,5首歌
歌曲保存路径:
    当前路径下的%s文件夹内
************************************************************'''


'''音乐下载器'''
class musicdl():
    def __init__(self, configpath=None, config=None, **kwargs):
        assert configpath or config, 'configpath of config should be given...'
        self.config = loadConfig(configpath) if config is None else config
        self.logger_handle = Logger(self.config['logfilepath'])
        self.initializeAllSources()
    '''非开发人员外部调用-手动输入版'''
    def run(self, target_srcs=None):
        while True:
            print(BASICINFO % (__version__, self.config.get('savedir')))
            # 音乐搜索
            user_input = self.dealInput('请输入歌曲搜索的关键词: ')
            target_srcs = [
                'kugou', 'kuwo', 'qqmusic', 'qianqian', 'fivesing',
                'netease', 'migu', 'xiami', 'joox', 'yiting',
            ] if target_srcs is None else target_srcs
            search_results = self.search(user_input, target_srcs)
            # 打印搜索结果
            title = ['序号', '歌手', '歌名', '大小', '时长', '专辑', '来源']
            items, records, idx = [], {}, 0
            for key, values in search_results.items():
                for value in values:
                    items.append([str(idx), value['singers'], value['songname'], value['filesize'], value['duration'], value['album'], value['source']])
                    records.update({str(idx): value})
                    idx += 1
            printTable(title, items)
            # 音乐下载
            user_input = self.dealInput('请输入想要下载的音乐编号: ')
            need_download_numbers = user_input.replace(' ', '').split(',')
            songinfos = []
            for item in need_download_numbers:
                songinfo = records.get(item, '')
                if songinfo: songinfos.append(songinfo)
            self.download(songinfos)
    '''非开发人员外部调用-语音版'''
    def runbyspeech(self, target_srcs=None, baiduspeech_params=None):
        assert baiduspeech_params is not None, 'please visit to https://console.bce.baidu.com/ai/?fromai=1#/ai/speech/overview/index to obtain AppID, AppKey and SecretKey'
        sr_api = SpeechRecognition(**baiduspeech_params)
        while True:
            print(BASICINFO % (__version__, self.config.get('savedir')))
            # 音乐搜索
            sr_api.synthesisspeak('请问您想听的歌曲名是什么呢?')
            time.sleep(4)
            target_srcs = ['migu'] if target_srcs is None else target_srcs
            sr_api.record()
            user_input = sr_api.recognition()
            self.logger_handle.info(f'识别结果为: {user_input}')
            search_results = self.search(user_input, target_srcs)
            # 音乐下载
            songinfos, songpaths = [], []
            for key, values in search_results.items():
                for value in values:
                    songinfos.append(value)
                    songpaths.append(os.path.join(value['savedir'], value['savename']+'.'+value['ext']))
            sr_api.synthesisspeak(f'共搜索到{len(songinfos)}首和{user_input}相关的歌曲, 将依次为您下载播放.')
            self.download(songinfos)
            # 音乐播放
            for songpath in songpaths:
                sr_api.synthesisspeak(audiopath=songpath)
    '''音乐搜索'''
    def search(self, keyword, target_srcs):
        def threadSearch(search_api, keyword, target_src, search_results):
            try:
                search_results.update({target_src: search_api(keyword)})
            except Exception as err:
                self.logger_handle.error(str(err), True)
                self.logger_handle.warning('无法在%s中搜索 ——> %s...' % (target_src, keyword))
        task_pool, search_results = [], {}
        for target_src in target_srcs:
            task = threading.Thread(
                target=threadSearch,
                args=(getattr(self, target_src).search, keyword, target_src, search_results)
            )
            task_pool.append(task)
            task.start()
        for task in task_pool:
            task.join()
        return search_results
    '''音乐下载'''
    def download(self, songinfos):
        for songinfo in songinfos:
            getattr(self, songinfo['source']).download([songinfo])
    '''初始化所有支持的搜索/下载源'''
    def initializeAllSources(self):
        supported_sources = {
            'kuwo': Kuwo,
            'joox': Joox,
            'migu': Migu,
            'kugou': Kugou,
            'lizhi': Lizhi,
            'xiami': Xiami,
            'yiting': YiTing,
            'netease': Netease,
            'qqmusic': QQMusic,
            'qianqian': Qianqian,
            'fivesing': FiveSing,
        }
        for key, value in supported_sources.items():
            setattr(self, key, value(copy.deepcopy(self.config), self.logger_handle))
    '''处理用户输入'''
    def dealInput(self, tip=''):
        user_input = input(tip)
        if user_input.lower() == 'q':
            self.logger_handle.info('ByeBye...')
            sys.exit()
        elif user_input.lower() == 'r':
            self.initializeAllSources()
            self.run()
        else:
            return user_input


'''run'''
if __name__ == '__main__':
    dl_client = musicdl('config.json')
    dl_client.run()


'''initialize'''
from .kuwo import Kuwo
from .migu import Migu
from .joox import Joox
from .lizhi import Lizhi
from .xiami import Xiami
from .kugou import Kugou
from .yiting import YiTing
from .netease import Netease
from .qqmusic import QQMusic
from .qianqian import Qianqian
from .fivesing import FiveSing


'''
Function:
    JOOX音乐下载: https://www.joox.com/cn/login
Author:
    Charles
微信公众号:
    Charles的皮卡丘
'''
import json
import time
import base64
import requests
from .base import Base
from ..utils.misc import *


'''JOOX音乐下载类'''
class Joox(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(Joox, self).__init__(config, logger_handle, **kwargs)
        self.source = 'joox'
        self.__initialize()
    '''歌曲搜索'''
    def search(self, keyword):
        self.logger_handle.info('正在%s中搜索 ——> %s...' % (self.source, keyword))
        cfg = self.config.copy()
        params = {
            'country': 'hk',
            'lang': 'zh_TW',
            'search_input': keyword,
            'sin': '0',
            'ein': cfg['search_size_per_source']
        }
        response = self.session.get(self.search_url, headers=self.headers, params=params)
        all_items = response.json()['itemlist']
        songinfos = []
        for item in all_items:
            params = {
                'songid': item['songid'],
                'lang': 'zh_cn',
                'country': 'hk',
                'from_type': '-1',
                'channel_id': '-1',
                '_': str(int(time.time()*1000))
            }
            response = self.session.get(self.songinfo_url, headers=self.headers, params=params)
            response_json = json.loads(response.text.replace('MusicInfoCallback(', '')[:-1])
            if response_json.get('code') != 0: continue
            for q_key in [('r320Url', '320'), ('r192Url', '192'), ('mp3Url', '128')]:
                download_url = response_json.get(q_key[0], '')
                if not download_url: continue
                filesize = str(round(int(json.loads(response_json['kbps_map'])[q_key[1]])/1024/1024, 2)) + 'MB'
                ext = 'mp3' if q_key[0] in ['r320Url', 'mp3Url'] else 'm4a'
            if not download_url: continue
            params = {
                'musicid': str(item['songid']),
                'country': 'hk',
                'lang': 'zh_cn',
            }
            response = self.session.get(self.lyric_url, headers=self.lyric_headers, params=params)
            lyric = base64.b64decode(response.json().get('lyric', '')).decode('utf-8')
            duration = int(item.get('playtime', 0))
            songinfo = {
                'source': self.source,
                'songid': str(item['songid']),
                'singers': filterBadCharacter(','.join([base64.b64decode(s['name']).decode('utf-8') for s in item.get('singer_list', [])])),
                'album': filterBadCharacter(response_json.get('malbum', '-')),
                'songname': filterBadCharacter(response_json.get('msong', '-')),
                'savedir': cfg['savedir'],
                'savename': '_'.join([self.source, filterBadCharacter(response_json.get('msong', '-'))]),
                'download_url': download_url,
                'lyric': lyric,
                'filesize': filesize,
                'ext': ext,
                'duration': seconds2hms(duration)
            }
            if not songinfo['album']: songinfo['album'] = '-'
            songinfos.append(songinfo)
        return songinfos
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_5) AppleWebKit/605.1.15 (KHTML, like Gecko)',
            'Cookie': 'wmid=142420656; user_type=1; country=id; session_key=2a5d97d05dc8fe238150184eaf3519ad;',
            'X-Forwarded-For': '36.73.34.109'
        }
        self.lyric_headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36',
            'Origin': 'https://www.joox.com'
        }
        self.search_url = 'https://api-jooxtt.sanook.com/web-fcgi-bin/web_search'
        self.songinfo_url = 'https://api.joox.com/web-fcgi-bin/web_get_songinfo'
        self.lyric_url = 'https://api-jooxtt.sanook.com/web-fcgi-bin/web_lyric'


'''
Function:
    千千音乐下载: http://music.taihe.com/
Author:
    Charles
微信公众号:
    Charles的皮卡丘
'''
import time
import hashlib
import requests
from .base import Base
from ..utils.misc import *


'''千千音乐下载类'''
class Qianqian(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(Qianqian, self).__init__(config, logger_handle, **kwargs)
        self.source = 'qianqian'
        self.__initialize()
    '''歌曲搜索'''
    def search(self, keyword):
        self.logger_handle.info('正在%s中搜索 ——> %s...' % (self.source, keyword))
        cfg = self.config.copy()
        params = {
            'sign': self.__calcSign(keyword, '16073360'),
            'word': keyword,
            'timestamp': str(int(time.time())),
            'appid': '16073360',
        }
        response = self.session.get(self.search_url, headers=self.headers, params=params)
        all_items = response.json()['data']['typeTrack']
        songinfos = []
        for item in all_items:
            params = {
                'sign': self.__calcSign(keyword, '16073360', item['TSID']),
                'TSID': item['TSID'],
                'timestamp': str(int(time.time())),
                'appid': '16073360',
            }
            response = self.session.get(self.tracklink_url, headers=self.headers, params=params)
            response_json = response.json()
            if response_json.get('errno') != 22000: continue
            if 'path' in response_json['data']:
                download_url = response_json['data']['path']
            else:
                download_url = response_json['data']['trail_audio_info']['path']
            if not download_url: continue
            lyric_url, lyric = response_json['data'].get('lyric', ''), ''
            if lyric_url:
                response = self.session.get(lyric_url, headers=self.headers)
                response.encoding = 'utf-8'
                lyric = response.text
            filesize = str(round(int(response_json['data']['size'])/1024/1024, 2)) + 'MB'
            ext = response_json['data']['format']
            duration = int(response_json['data']['duration'])
            songinfo = {
                'source': self.source,
                'songid': str(item['id']),
                'singers': filterBadCharacter(item['artist'][0].get('name', '-')),
                'album': filterBadCharacter(item.get('albumTitle', '-')),
                'songname': filterBadCharacter(item.get('title', '-')).split('–')[0].strip(),
                'savedir': cfg['savedir'],
                'savename': '_'.join([self.source, filterBadCharacter(item.get('title', '-')).split('–')[0].strip()]),
                'download_url': download_url,
                'lyric': lyric,
                'filesize': filesize,
                'ext': ext,
                'duration': seconds2hms(duration)
            }
            if not songinfo['album']: songinfo['album'] = '-'
            songinfos.append(songinfo)
            if len(songinfos) == cfg['search_size_per_source']: break
        return songinfos
    '''计算sign值'''
    def __calcSign(self, keyword, appid, TSID=None):
        secret = '0b50b02fd0d73a9c4c8c3a781c30845f'
        if TSID is None:
            e = {
                'word': keyword,
                'appid': appid,
                'timestamp': str(int(time.time()))
            }
            n = list(e.keys())
            n.sort()
            i = f'{n[0]}={e[n[0]]}'
            for r in range(1, len(n)):
                o = n[r]
                i += f'&{o}={e[o]}'
            sign = hashlib.md5((i + secret).encode('utf-8')).hexdigest()
        else:
            sign = hashlib.md5((f'TSID={TSID}&appid={appid}&timestamp={str(int(time.time()))}{secret}').encode('utf-8')).hexdigest()
        return sign
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.82 Safari/537.36',
            'Referer': 'https://music.taihe.com/'
        }
        self.search_url = 'https://music.taihe.com/v1/search'
        self.tracklink_url = 'https://music.taihe.com/v1/song/tracklink'


'''
Function:
    5SING音乐下载: http://5sing.kugou.com/
Author:
    Charles
微信公众号:
    Charles的皮卡丘
'''
import re
import requests
from .base import Base
from ..utils.misc import *


'''5SING音乐下载类'''
class FiveSing(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(FiveSing, self).__init__(config, logger_handle, **kwargs)
        self.source = 'fivesing'
        self.__initialize()
    '''歌曲搜索'''
    def search(self, keyword):
        self.logger_handle.info('正在%s中搜索 ——> %s...' % (self.source, keyword))
        cfg = self.config.copy()
        response = self.session.get(self.search_url+keyword, headers=self.headers)
        response.encoding = 'uft-8'
        all_items = re.findall(r"dataList = '(.*?)';", response.text)[0]
        all_items = all_items.replace(r'<em class=\\\"keyword\\\">', '')
        all_items = all_items.replace(r'<\\/em>', '')
        all_items = eval(all_items.replace(r'\"', '"'))
        songinfos = []
        for item in all_items:
            try:
                item['songName'] = item['songName'].encode('utf-8').decode('unicode_escape')
            except:
                item['songName'] = '解码失败: ' + item['songName']
            try:
                item['singer'] = item['singer'].encode('utf-8').decode('unicode_escape')
            except:
                item['singer'] = '解码失败: ' + item['singer']
            params = {
                'songid': str(item['songId']),
                'songtype': 'yc' if 'yc' in item['downloadurl'] else 'fc'
            }
            response = self.session.get(self.songinfo_url, headers=self.headers, params=params)
            response_json = response.json()
            if response_json.get('code') != 1000: continue
            for quality in ['sq', 'hq', 'lq']:
                download_url = response_json.get('data', {}).get(f'{quality}url', '')
                if download_url: break
            if not download_url: continue
            filesize = str(round(int(response_json['data'][f'{quality}size'])/1024/1024, 2)) + 'MB'
            ext = response_json['data'][f'{quality}ext']
            params = {
                'songtype': 'yc' if 'yc' in item['downloadurl'] else 'fc',
                'songid': str(item['songId']),
                'songfields': '',
                'userfields': '',
            }
            response = self.session.get(self.lyric_url, headers=self.headers, params=params)
            response_json = response.json()
            lyric = response_json.get('data', {}).get('dynamicWords', '')
            duration = '-:-:-'
            songinfo = {
                'source': self.source,
                'songid': str(item['songId']),
                'singers': filterBadCharacter(item.get('singer', '-')),
                'album': filterBadCharacter(response_json['data'].get('albumName', '-')),
                'songname': filterBadCharacter(item.get('songName', '-')),
                'savedir': cfg['savedir'],
                'savename': '_'.join([self.source, filterBadCharacter(item.get('songName', '-'))]),
                'download_url': download_url,
                'lyric': lyric,
                'filesize': filesize,
                'ext': ext,
                'duration': duration
            }
            if not songinfo['album']: songinfo['album'] = '-'
            songinfos.append(songinfo)
            if len(songinfos) > cfg['search_size_per_source']: break
        return songinfos
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36',
        }
        self.search_url = 'http://search.5sing.kugou.com/?keyword='
        self.songinfo_url = 'http://mobileapi.5sing.kugou.com/song/getSongUrl'
        self.lyric_url = 'http://mobileapi.5sing.kugou.com/song/newget'


'''
Function:
    咪咕音乐下载: http://www.migu.cn/
Author:
    Charles
微信公众号:
    Charles的皮卡丘
'''
import requests
from .base import Base
from ..utils.misc import *


'''咪咕音乐下载类'''
class Migu(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(Migu, self).__init__(config, logger_handle, **kwargs)
        self.source = 'migu'
        self.__initialize()
    '''歌曲搜索'''
    def search(self, keyword):
        self.logger_handle.info('正在%s中搜索 ——> %s...' % (self.source, keyword))
        cfg = self.config.copy()
        params = {
            'ua': 'Android_migu',
            'version': '5.0.1',
            'text': keyword,
            'pageNo': '1',
            'pageSize': cfg['search_size_per_source'],
            'searchSwitch': '{"song":1,"album":0,"singer":0,"tagSong":0,"mvSong":0,"songlist":0,"bestShow":1}',
        }
        response = self.session.get(self.search_url, headers=self.headers, params=params)
        all_items = response.json()['songResultData']['result']
        songinfos = []
        for item in all_items:
            ext = ''
            download_url = ''
            filesize = '-MB'
            for rate in sorted(item.get('rateFormats', []), key=lambda x: int(x['size']), reverse=True):
                if (int(rate['size']) == 0) or (not rate.get('formatType', '')) or (not rate.get('resourceType', '')): continue
                ext = 'flac' if rate.get('formatType') == 'SQ' else 'mp3'
                download_url = self.player_url.format(
                    copyrightId=item['copyrightId'], 
                    contentId=item['contentId'], 
                    toneFlag=rate['formatType'],
                    resourceType=rate['resourceType']
                )
                filesize = str(round(int(rate['size'])/1024/1024, 2)) + 'MB'
                break
            if not download_url: continue
            lyric_url, lyric = item.get('lyricUrl', ''), ''
            if lyric_url:
                response = self.session.get(lyric_url, headers=self.headers)
                response.encoding = 'utf-8'
                lyric = response.text
            duration = '-:-:-'
            songinfo = {
                'source': self.source,
                'songid': str(item['id']),
                'singers': filterBadCharacter(','.join([s.get('name', '') for s in item.get('singers', [])])),
                'album': filterBadCharacter(item.get('albums', [{'name': '-'}])[0].get('name', '-')),
                'songname': filterBadCharacter(item.get('name', '-')),
                'savedir': cfg['savedir'],
                'savename': '_'.join([self.source, filterBadCharacter(item.get('name', '-'))]),
                'download_url': download_url,
                'lyric': lyric,
                'filesize': filesize,
                'ext': ext,
                'duration': duration
            }
            if not songinfo['album']: songinfo['album'] = '-'
            songinfos.append(songinfo)
            if len(songinfos) == cfg['search_size_per_source']: break
        return songinfos
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'Referer': 'https://m.music.migu.cn/', 
            'User-Agent': 'Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/73.0.3683.75 Mobile Safari/537.36'
        }
        self.search_url = 'http://pd.musicapp.migu.cn/MIGUM3.0/v1.0/content/search_all.do'
        self.player_url = 'https://app.pd.nf.migu.cn/MIGUM3.0/v1.0/content/sub/listenSong.do?channel=mx&copyrightId={copyrightId}&contentId={contentId}&toneFlag={toneFlag}&resourceType={resourceType}&userId=15548614588710179085069&netType=00'


'''
Function:
    虾米音乐下载: https://www.xiami.com/
Author:
    Charles
微信公众号:
    Charles的皮卡丘
'''
import re
import time
import json
import requests
from .base import Base
from hashlib import md5
from ..utils.misc import *


'''虾米音乐下载类'''
class Xiami(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(Xiami, self).__init__(config, logger_handle, **kwargs)
        self.source = 'xiami'
        self.__initialize()
    '''歌曲搜索'''
    def search(self, keyword):
        self.logger_handle.info('正在%s中搜索 ——> %s...' % (self.source, keyword))
        cfg = self.config.copy()
        token = self.__getToken()
        search_url = self.base_url.format(action=self.actions['searchsongs'])
        params = {
            'key': keyword,
            'pagingVO': {'page': '1', 'pageSize': str(cfg['search_size_per_source'])}
        }
        response = self.session.get(search_url, headers=self.headers, params=self.__xiamiSign(params, token))
        all_items = response.json()['data']['data']['songs']
        songinfos = []
        for item in all_items:
            download_url = ''
            for file_info in item['listenFiles']:
                if not file_info['downloadFileSize']: continue
                filesize = str(round(int(file_info['downloadFileSize'])/1024/1024, 2)) + 'MB'
                download_url = file_info['listenFile']
                ext = file_info['format']
                duration = int(file_info.get('length', 0)) / 1000
                break
            if not download_url: continue
            lyric_url, lyric = item.get('lyricInfo', {}).get('lyricFile', ''), ''
            if lyric_url:
                response = self.session.get(lyric_url, headers=self.headers)
                response.encoding = 'utf-8'
                lyric = response.text
            songinfo = {
                'source': self.source,
                'songid': str(item['songId']),
                'singers': filterBadCharacter(item.get('artistName', '-')),
                'album': filterBadCharacter(item.get('albumName', '-')),
                'songname': filterBadCharacter(item.get('songName', '-')).split('–')[0].strip(),
                'savedir': cfg['savedir'],
                'savename': '_'.join([self.source, filterBadCharacter(item.get('songName', '-')).split('–')[0].strip()]),
                'download_url': download_url,
                'lyric': lyric,
                'filesize': filesize,
                'ext': ext,
                'duration': seconds2hms(duration)
            }
            if not songinfo['album']: songinfo['album'] = '-'
            songinfos.append(songinfo)
        return songinfos
    '''虾米签名'''
    def __xiamiSign(self, params, token=''):
        appkey = '23649156'
        t = str(int(time.time() * 1000))
        request_str = {
            'header': {'appId': '200', 'platformId': 'h5'},
            'model': params
        }
        data = json.dumps({'requestStr': json.dumps(request_str)})
        sign = '%s&%s&%s&%s' % (token, t, appkey, data)
        sign = md5(sign.encode('utf-8')).hexdigest()
        params = {
            't': t,
            'appKey': appkey,
            'sign': sign,
            'data': data
        }
        return params
    '''获得请求所需的token'''
    def __getToken(self):
        action = self.actions['getsongdetail']
        url = self.base_url.format(action=action)
        params = {'songId': '1'}
        response = self.session.get(url, params=self.__xiamiSign(params))
        cookies = response.cookies.get_dict()
        return cookies['_m_h5_tk'].split('_')[0]
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.120 Safari/537.36',
            'Referer': 'http://h.xiami.com',
            'Connection': 'keep-alive',
            'Accept-Language': 'zh-CN,zh;q=0.8,gl;q=0.6,zh-TW;q=0.4',
            'Accept-Encoding': 'gzip,deflate,sdch',
            'Accept': '*/*'
        }
        self.base_url = 'https://acs.m.xiami.com/h5/{action}/1.0/'
        self.actions = {
            'searchsongs': 'mtop.alimusic.search.searchservice.searchsongs',
            'getsongdetail': 'mtop.alimusic.music.songservice.getsongdetail',
            'getsongs': 'mtop.alimusic.music.songservice.getsongs',
            'getsonglyrics': 'mtop.alimusic.music.lyricservice.getsonglyrics'
        }


'''
Function:
    一听音乐下载: https://h5.1ting.com/
Author:
    Charles
微信公众号:
    Charles的皮卡丘
'''
import time
import requests
from .base import Base
from ..utils.misc import *


'''一听音乐下载类'''
class YiTing(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(YiTing, self).__init__(config, logger_handle, **kwargs)
        self.source = 'yiting'
        self.__initialize()
    '''歌曲搜索'''
    def search(self, keyword):
        self.logger_handle.info('正在%s中搜索 ——> %s...' % (self.source, keyword))
        cfg = self.config.copy()
        params = {
            'q': keyword,
            'page': '1',
            'size': cfg['search_size_per_source'],
        }
        response = self.session.get(self.search_url, headers=self.headers, params=params)
        all_items = response.json()['results']
        songinfos = []
        for item in all_items:
            params = {
                'ids': item['song_id']
            }
            self.headers.update({'Referer': f'http://h5.1ting.com/{item["song_id"]}/song/'})
            response = self.session.get(self.songinfo_url, headers=self.headers, params=params)
            response_json = response.json()[0]
            if 'song_filepath' not in response_json: continue
            download_url = 'http://h5.1ting.com/file?url=' + response_json['song_filepath'].replace('.wma', '.mp3')
            self.headers.update({'Referer': f'http://www.1ting.com/geci{item["song_id"]}.html'})
            response = self.session.get(self.lyric_url+str(item['song_id']), headers=self.headers)
            response.encoding = 'utf-8'
            lyric = response.text
            filesize = '-MB'
            ext = download_url.split('.')[-1]
            duration = '-:-:-'
            songinfo = {
                'source': self.source,
                'songid': str(item['song_id']),
                'singers': filterBadCharacter(item.get('singer_name', '-')),
                'album': filterBadCharacter(item.get('album_name', '-')),
                'songname': filterBadCharacter(item.get('song_name', '-')),
                'savedir': cfg['savedir'],
                'savename': '_'.join([self.source, filterBadCharacter(item.get('song_name', '-'))]),
                'download_url': download_url,
                'lyric': lyric,
                'filesize': filesize,
                'ext': ext,
                'duration': duration
            }
            if not songinfo['album']: songinfo['album'] = '-'
            songinfos.append(songinfo)
            if len(songinfos) == cfg['search_size_per_source']: break
        return songinfos
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.198 Safari/537.36'
        }
        self.search_url = 'http://so.1ting.com/song/json'
        self.songinfo_url = 'http://h5.1ting.com/touch/api/song'
        self.lyric_url = 'http://www.1ting.com/api/geci/lrc/'


'''
Function:
    语音识别模块
Author:
    Charles
微信公众号:
    Charles的皮卡丘
'''
import os
import time


'''语音识别模块'''
class SpeechRecognition():
    def __init__(self, app_id, api_key, secret_key, **kwargs):
        from aip import AipSpeech
        self.aipspeech_api = AipSpeech(app_id, api_key, secret_key)
        self.speech_path = kwargs.get('speech_path', 'recording.wav')
        assert self.speech_path.endswith('.wav'), 'only support audio with wav format'
    '''录音'''
    def record(self, sample_rate=16000):
        import speech_recognition as sr
        rec = sr.Recognizer()
        with sr.Microphone(sample_rate=sample_rate) as source:
            audio = rec.listen(source)
        with open(self.speech_path, 'wb') as fp:
            fp.write(audio.get_wav_data())
    '''识别'''
    def recognition(self):
        assert os.path.exists(self.speech_path)
        with open(self.speech_path, 'rb') as fp:
            content = fp.read()
        result = self.aipspeech_api.asr(content, 'wav', 16000, {'dev_pid': 1536})
        text = result['result'][0]
        return text
    '''合成并说话'''
    def synthesisspeak(self, text=None, audiopath=None):
        assert text is None or audiopath is None
        import pygame
        if audiopath is None:
            audiopath = f'recording_{time.time()}.mp3'
            result = self.aipspeech_api.synthesis(
                text, 'zh', 1, 
                {'spd': 4, 'vol': 5, 'per': 4}
            )
            if not isinstance(result, dict):
                with open(audiopath, 'wb') as fp:
                    fp.write(result)
            pygame.mixer.init()
            pygame.mixer.music.load(audiopath)
            pygame.mixer.music.play()
        else:
            pygame.mixer.init()
            pygame.mixer.music.load(audiopath)
            pygame.mixer.music.play()
            while pygame.mixer.music.get_busy():
                time.sleep(15)


'''
Function:
    网易云音乐下载: https://music.163.com/
Author:
    Charles
微信公众号:
    Charles的皮卡丘
'''
import os
import base64
import codecs
import requests
from .base import Base
from ..utils.misc import *
from Crypto.Cipher import AES


'''用于算post的两个参数, 具体原理详见知乎：https://www.zhihu.com/question/36081767'''
class Cracker():
    def __init__(self):
        self.modulus = '00e0b509f6259df8642dbc35662901477df22677ec152b5ff68ace615bb7b725152b3ab17a876aea8a5aa76d2e417629ec4ee341f56135fccf695280104e0312ecbda92557c93870114af6c9d05c4f7f0c3685b7a46bee255932575cce10b424d813cfe4875d3e82047b97ddef52741d546b8e289dc6935b3ece0462db0a22b8e7'
        self.nonce = '0CoJUm6Qyw8W8jud'
        self.pubKey = '010001'
    def get(self, text):
        text = json.dumps(text)
        secKey = self.__createSecretKey(16)
        encText = self.__aesEncrypt(self.__aesEncrypt(text, self.nonce), secKey)
        encSecKey = self.__rsaEncrypt(secKey, self.pubKey, self.modulus)
        post_data = {
            'params': encText,
            'encSecKey': encSecKey
        }
        return post_data
    def __aesEncrypt(self, text, secKey):
        pad = 16 - len(text) % 16
        if isinstance(text, bytes):
            text = text.decode('utf-8')
        text = text + str(pad * chr(pad))
        secKey = secKey.encode('utf-8')
        encryptor = AES.new(secKey, 2, b'0102030405060708')
        text = text.encode('utf-8')
        ciphertext = encryptor.encrypt(text)
        ciphertext = base64.b64encode(ciphertext)
        return ciphertext
    def __rsaEncrypt(self, text, pubKey, modulus):
        text = text[::-1]
        rs = int(codecs.encode(text.encode('utf-8'), 'hex_codec'), 16) ** int(pubKey, 16) % int(modulus, 16)
        return format(rs, 'x').zfill(256)
    def __createSecretKey(self, size):
        return (''.join(map(lambda xx: (hex(ord(xx))[2:]), str(os.urandom(size)))))[0: 16]


'''网易云音乐下载类'''
class Netease(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(Netease, self).__init__(config, logger_handle, **kwargs)
        self.source = 'netease'
        self.cracker = Cracker()
        self.__initialize()
    '''歌曲搜索'''
    def search(self, keyword):
        self.logger_handle.info('正在%s中搜索 ——> %s...' % (self.source, keyword))
        cfg = self.config.copy()
        params = {
            's': keyword,
            'type': '1',
            'offset': '0',
            'sub': 'false',
            'limit': cfg['search_size_per_source']
        }
        response = self.session.post(self.search_url, headers=self.headers, params=params, data=self.cracker.get(params))
        all_items = response.json()['result']['songs']
        songinfos = []
        for item in all_items:
            if item['privilege']['fl'] == 0: continue
            for q in ['h', 'm', 'l']:
                params = {
                    'ids': [item['id']],
                    'br': item[q]['br'],
                    'csrf_token': ''
                }
                response = self.session.post(self.player_url, headers=self.headers, data=self.cracker.get(params))
                response_json = response.json()
                if response_json.get('code') == 200: break
            if response_json.get('code') != 200: continue
            download_url = response_json['data'][0]['url']
            if not download_url: continue
            params = {
                'csrf_token': '',
                'id': item['id'],
                'lv': '-1',
                'tv': '-1'
            }
            response = self.session.post(self.lyric_url, headers=self.headers, data=self.cracker.get(params))
            lyric = response.json().get('lrc', {}).get('lyric', '')
            filesize = str(round(int(item[q]['size'])/1024/1024, 2)) + 'MB'
            ext = download_url.split('.')[-1]
            duration = int(item.get('dt', 0) / 1000)
            songinfo = {
                'source': self.source,
                'songid': str(item['id']),
                'singers': filterBadCharacter(','.join([s.get('name', '') for s in item.get('ar')])),
                'album': filterBadCharacter(item.get('al', {}).get('name', '-')),
                'songname': filterBadCharacter(item.get('name', '-')),
                'savedir': cfg['savedir'],
                'savename': '_'.join([self.source, filterBadCharacter(item.get('name', '-'))]),
                'download_url': download_url,
                'lyric': lyric,
                'filesize': filesize,
                'ext': ext,
                'duration': seconds2hms(duration)
            }
            if not songinfo['album']: songinfo['album'] = '-'
            songinfos.append(songinfo)
        return songinfos
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'Accept': '*/*',
            'Accept-Encoding': 'gzip,deflate,sdch',
            'Accept-Language': 'zh-CN,zh;q=0.8,gl;q=0.6,zh-TW;q=0.4',
            'Connection': 'keep-alive',
            'Content-Type': 'application/x-www-form-urlencoded',
            'Host': 'music.163.com',
            'Origin': 'https://music.163.com',
            'Referer': 'https://music.163.com/',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/65.0.3325.32 Safari/537.36'
        }
        self.search_url = 'http://music.163.com/weapi/cloudsearch/get/web?csrf_token='
        self.player_url = 'http://music.163.com/weapi/song/enhance/player/url?csrf_token='
        self.lyric_url = 'https://music.163.com/weapi/song/lyric'


'''title'''
__title__ = 'musicdl'
'''description'''
__description__ = 'A lightweight music downloader written by pure python'
'''url'''
__url__ = 'https://github.com/CharlesPikachu/musicdl'
'''version'''
__version__ = '2.2.4'
'''author'''
__author__ = 'Charles'
'''email'''
__email__ = 'charlesblwx@gmail.com'
'''license'''
__license__ = 'MIT License'
'''copyright'''
__copyright__ = 'Copyright 2020 Charles'


'''import all'''
from .downloader import Downloader
from .speech import SpeechRecognition
from .logger import Logger, printTable
from .misc import checkDir, seconds2hms, loadConfig, filterBadCharacter


'''
Function:
    酷我音乐下载: http://www.kuwo.cn/
Author:
    Charles
微信公众号:
    Charles的皮卡丘
'''
import time
import requests
from .base import Base
from ..utils.misc import *


'''酷我音乐下载类'''
class Kuwo(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(Kuwo, self).__init__(config, logger_handle, **kwargs)
        self.source = 'kuwo'
        self.__initialize()
    '''歌曲搜索'''
    def search(self, keyword):
        self.logger_handle.info('正在%s中搜索 ——> %s...' % (self.source, keyword))
        response = self.session.get('http://kuwo.cn/search/list?key=hello', headers=self.headers)
        cookies, token = response.cookies, response.cookies.get('kw_token')
        self.headers['csrf'] = token
        cfg = self.config.copy()
        params = {
            'key': keyword,
            'pn': '1',
            'rn': cfg['search_size_per_source'],
        }
        response = self.session.get(self.search_url, headers=self.headers, params=params, cookies=cookies)
        all_items = response.json()['data']['list']
        songinfos = []
        for item in all_items:
            for br in ['320kmp3', '192kmp3', '128kmp3']:
                params = {
                    'format': 'mp3',
                    'br': br,
                    'rid': str(item['rid']),
                    'type': 'convert_url',
                    'response': 'url',
                }
                download_url = self.session.get(self.player_url, params=params).text
                if not (download_url.startswith('http://') or download_url.startswith('https://')): continue
                break
            if not (download_url.startswith('http://') or download_url.startswith('https://')): continue
            params = {
                'musicId': str(item['rid'])
            }
            self.lyric_headers.update({'Referer': f'http://m.kuwo.cn/yinyue/{str(item["rid"])}'})
            response = self.session.get(self.lyric_url, headers=self.lyric_headers, params=params)
            lyric = response.json().get('data', {}).get('lrclist', '')
            filesize = '-MB'
            ext = download_url.split('.')[-1]
            duration = int(item.get('duration', 0))
            songinfo = {
                'source': self.source,
                'songid': str(item['rid']),
                'singers': filterBadCharacter(item.get('artist', '-')),
                'album': filterBadCharacter(item.get('album', '-')),
                'songname': filterBadCharacter(item.get('name', '-')),
                'savedir': cfg['savedir'],
                'savename': '_'.join([self.source, filterBadCharacter(item.get('name', '-'))]),
                'download_url': download_url,
                'lyric': lyric,
                'filesize': filesize,
                'ext': ext,
                'duration': seconds2hms(duration)
            }
            if not songinfo['album']: songinfo['album'] = '-'
            songinfos.append(songinfo)
        return songinfos
    '''初始化'''
    def __initialize(self):
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36',
            'Accept': '*/*',
            'Accept-Encoding': 'gzip,deflate',
            'Accept-Language': 'zh-CN,zh;q=0.8,gl;q=0.6,zh-TW;q=0.4',
            'Host': 'kuwo.cn',
            'Referer': 'http://kuwo.cn',
        }
        self.lyric_headers = {
            'User-Agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 9_1 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13B143 Safari/601.1',
            'Referer': 'http://m.kuwo.cn/yinyue/'
        }
        self.search_url = 'http://www.kuwo.cn/api/www/search/searchMusicBykeyWord'
        self.player_url = 'http://antiserver.kuwo.cn/anti.s'
        self.lyric_url = 'http://m.kuwo.cn/newh5/singles/songinfoandlrc'


'''
Function:
    下载器类
Author:
    Charles
微信公众号:
    Charles的皮卡丘
'''
import click
import warnings
import requests
from .misc import *
warnings.filterwarnings('ignore')


'''下载器类'''
class Downloader():
    def __init__(self, songinfo, session=None, **kwargs):
        self.songinfo = songinfo
        self.session = requests.Session() if session is None else session
        self.__setheaders(songinfo['source'])
    '''外部调用'''
    def start(self):
        songinfo, session, headers = self.songinfo, self.session, self.headers
        checkDir(songinfo['savedir'])
        try:
            is_success = False
            with session.get(songinfo['download_url'], headers=headers, stream=True, verify=False) as response:
                if response.status_code == 200:
                    total_size, chunk_size = int(response.headers['content-length']), 1024
                    label = '[FileSize]: %0.2fMB' % (total_size / 1024 / 1024)
                    with click.progressbar(length=total_size, label=label) as progressbar:
                        with open(os.path.join(songinfo['savedir'], songinfo['savename']+'.'+songinfo['ext']), 'wb') as fp:
                            for chunk in response.iter_content(chunk_size=chunk_size):
                                if chunk:
                                    fp.write(chunk)
                                    progressbar.update(len(chunk))
                    is_success = True
        except:
            is_success = False
        return is_success
    '''设置请求头'''
    def __setheaders(self, source):
        self.qq_headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36',
            'Referer': 'http://y.qq.com',
        }
        self.migu_headers = {
            'Referer': 'https://m.music.migu.cn/', 
            'User-Agent': 'Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/73.0.3683.75 Mobile Safari/537.36',
        }
        self.kuwo_headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36',
        }
        self.joox_headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_5) AppleWebKit/605.1.15 (KHTML, like Gecko)',
        }
        self.xiami_headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.120 Safari/537.36',
        }
        self.kugou_headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36',
            'Host': 'webfs.yun.kugou.com',
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3',
            'Accept-Encoding': 'gzip, deflate, br',
            'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8',
            'Cache-Control': 'max-age=0',
            'Connection': 'keep-alive',
        }
        self.netease_headers = {}
        self.qianqian_headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36',
        }
        self.baiduFlac_headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36',
            'Referer': 'http://music.baidu.com/',
        }
        self.yiting_headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.198 Safari/537.36'
        }
        self.fivesing_headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.198 Safari/537.36'
        }
        self.lizhi_headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.198 Safari/537.36'
        }
        self.headers = getattr(self, f'{source}_headers')


'''
Function:
    酷狗音乐下载: http://www.kugou.com/
Author:
    Charles
微信公众号:
    Charles的皮卡丘
'''
import time
import requests
from .base import Base
from ..utils.misc import *


'''酷狗音乐下载类'''
class Kugou(Base):
    def __init__(self, config, logger_handle, **kwargs):
        super(Kugou, self).__init__(config, logger_handle, **kwargs)
        self.source = 'kugou'
        self.__initialize()
    '''歌曲搜索'''
    def search(self, keyword):
        self.logger_handle.info('正在%s中搜索 ——> %s...' % (self.source, keyword))
        cfg = self.config.copy()
        params = {
            'keyword': keyword,
            'page': '1',
            'pagesize': cfg['search_size_per_source'],
            'userid': '-1',
            'clientver': '',
            'platform': 'WebFilter',
            'tag': 'em',
            'filter': '',
            'iscorrection': '1',
            'privilege_filter': '0',
            '_': str(int(time.time() * 1000))
        }
        response = self.session.get(self.search_url, headers=self.search_headers, params=params)
        all_items = response.json()['data']['lists']
        songinfos = []
        for item in all_items:
            params = {
                'r': 'play/getdata',
                'hash': str(item['FileHash']),
                'album_id': str(item['AlbumID']),
                'dfid': '1aAcF31Utj2l0ZzFPO0Yjss0',
                'mid': 'ccbb9592c3177be2f3977ff292e0f145',
                'platid': '4',
                '_': str(int(time.time() * 1000))
            }
            response = self.session.get(self.hash_url, headers=self.hash_headers, params=params)
            response_json = response.json()
            if response_json.get('err_code') != 0: continue
            download_url = response_json['data']['play_url'].replace('\\', '')
            if not download_url: continue
            params = {
                'cmd': '100',
                'timelength': '999999',
                'hash': item.get('FileHash', '')
            }
            self.lyric_headers.update({'Referer': f'http://m.kugou.com/play/info/{str(item["ID"])}'})
            response = self.session.get(self.lyric_url, headers=self.lyric_headers, params=params)
            response.encoding = 'utf-8'
            lyric = response.text
            filesize = str(round(int(response_json['data']['filesize'])/1024/1024, 2)) + 'MB'
            ext = download_url.split('.')[-1]
            duration = int(item.get('Duration', 0))
            songinfo = {
                'source': self.source,
                'songid': str(item['ID']),
                'singers': filterBadCharacter(item.get('SingerName', '-')),
                'album': filterBadCharacter(item.get('AlbumName', '-')),
                'songname': filterBadCharacter(item.get('SongName', '-')),
                'savedir': cfg['savedir'],
                'savename': '_'.join([self.source, filterBadCharacter(item.get('SongName', '-'))]),
                'download_url': download_url,
                'filesize': filesize,
                'lyric': lyric,
                'ext': ext,
                'duration': seconds2hms(duration)
            }
            if not songinfo['album']: songinfo['album'] = '-'
            songinfos.append(songinfo)
        return songinfos
    '''初始化'''
    def __initialize(self):
        self.search_headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36',
            'Referer': 'https://www.kugou.com/yy/html/search.html'
        }
        self.hash_headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.89 Safari/537.36',
            'Referer':'https://www.kugou.com/song/'
        }
        self.lyric_headers = {
            'User-Agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 9_1 like Mac OS X] AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13B143 Safari/601.1',
            'Referer': 'http://m.kugou.com/play/info/'
        }
        self.search_url = 'http://songsearch.kugou.com/song_search_v2'
        self.hash_url = 'https://wwwapi.kugou.com/yy/index.php'
        self.lyric_url = 'http://m.kugou.com/app/i/krc.php'


'''
Function:
    各平台音乐下载器基类
Author:
    Charles
微信公众号:
    Charles的皮卡丘
'''
import requests
from ..utils.downloader import Downloader


'''音乐下载器基类'''
class Base():
    def __init__(self, config, logger_handle, **kwargs):
        self.source = None
        self.session = requests.Session()
        self.session.proxies.update(config['proxies'])
        self.config = config
        self.logger_handle = logger_handle
    '''歌曲搜索'''
    def search(self, keyword):
        raise NotImplementedError('not be implemented...')
    '''歌曲下载'''
    def download(self, songinfos):
        for songinfo in songinfos:
            self.logger_handle.info('正在从%s下载 ——> %s...' % (self.source, songinfo['savename']))
            task = Downloader(songinfo, self.session)
            if task.start():
                self.logger_handle.info('成功从%s下载到了 ——> %s...' % (self.source, songinfo['savename']))
            else:
                self.logger_handle.info('无法从%s下载 ——> %s...' % (self.source, songinfo['savename']))
    '''初始化'''
    def __initialize(self):
        raise NotImplementedError('not be implemented...')
    '''返回类信息'''
    def __repr__(self):
        return 'Music Source: %s' % self.source